with
   Ada.Integer_Text_IO,
   Ada.Text_IO,
   Ada.Text_IO.Unbounded_IO,
   Ada.Strings.Unbounded,
   Ada.Characters.Handling,
   Ada.Text_IO.Text_Streams;

use Ada.Strings.Unbounded;

procedure sample1  is
   type Word is record
      text: Ada.Strings.Unbounded.Unbounded_String;
      count: Natural := 0;  -- Total number of occurrences of this word
   end record;

   type Word_Array is array(1 .. 1000) of Word;

   type Word_List is record
      words: Word_Array;        --  The unique words
      num_words: Natural := 0;   --  How many unique words seen so far
   end record;

   function is_delim(ch: Character) return Boolean is
   begin
      return
         (ch = ' ') or else
         (ch = Character'Val(10)) or else
         (ch = Character'Val(13)) or else
         (ch = Character'Val(9)) or else
         (ch = '.') or else
         (ch = ',') or else
         (ch = ':') or else
         (ch = Character'Val(35)) or else
         (ch = '(') or else
         (ch = ')') or else
         (ch = '/') or else
         (ch = '?') or else
         (ch = '!');
   end is_delim;

   function is_word_char(ch: Character) return Boolean is
   begin
      return
         Ada.Characters.Handling.Is_Letter(ch) or else
         Ada.Characters.Handling.Is_Digit(ch) or else
         (ch = '_') or else
         (ch = '-');
   end is_word_char;

   function is_consisting_of_all_word_char(str: Ada.Strings.Unbounded.Unbounded_string) return Boolean is
   begin
      for i in 1 .. Ada.Strings.Unbounded.Length(str) loop
         if not is_word_char(Ada.Strings.Unbounded.Element(str, i)) then
            return false;
         end if;
      end loop;
      return true;
   end is_consisting_of_all_word_char;

   function is_word(str: Ada.Strings.Unbounded.Unbounded_string) return Boolean is
   begin
      return
         (Ada.Strings.Unbounded.Length(str) > 0) and then
         Ada.Characters.Handling.Is_Letter(Ada.Strings.Unbounded.Element(str, 1)) and then
         is_consisting_of_all_word_char(str);
   end is_word;

   procedure add_word(wl: in out Word_List; str: Ada.Strings.Unbounded.Unbounded_String) is
   begin
      for i in 1 .. wl.num_words loop
         if str = wl.words(i).text then
            wl.words(i).count := wl.words(i).count + 1;
            return;
         end if;
      end loop;

      wl.num_words := wl.num_words + 1;
      wl.words(wl.num_words).text := str;
      wl.words(wl.num_words).count := 1;
   end add_word;

   procedure get_words(input: Ada.Text_IO.File_Type; wl: in out Word_List) is
      s: Ada.Strings.Unbounded.Unbounded_String;
   begin
      while not Ada.Text_IO.End_of_File(input) loop
         declare
            c: Character;
         begin
            if Ada.Text_IO.End_of_Line(input) then
               c := Character'Val(10);
               Ada.Text_IO.Skip_Line(input);
            else
               Ada.Text_IO.Get(input, c);
            end if;

            if is_delim(c) then
               if is_word(s) then
                  add_word(wl, s);
               end if;
               s := Ada.Strings.Unbounded.To_Unbounded_String("");
            else
               s := s & c;
            end if;
         exception
            when Ada.Text_IO.End_Error =>
               return;
         end;
      end loop;
   end get_words;

   procedure put_words(output: Ada.Text_IO.File_Type; wl: Word_List) is
   begin
      for i in 1 .. wl.num_words loop
         Ada.Integer_Text_IO.Put(output, wl.words(i).count);
         Ada.Text_IO.Unbounded_IO.Put(output, " " & wl.words(i).text);
         Ada.Text_IO.New_Line(output);
      end loop;
   end put_words;

   procedure sort_words(wl: in out Word_List) is
   begin
      for i in 1 .. wl.num_words loop
         declare
            smallest: Natural := i;
            temp: Word;
         begin
            for j in i + 1 .. wl.num_words loop
               if wl.words(j).text < wl.words(smallest).text then
                  smallest := j;
               end if;
            end loop;

            if smallest /= i then
               temp := wl.words(i);
               wl.words(i) := wl.words(smallest);
               wl.words(smallest) := temp;
            end if;
         end;
      end loop;
   end sort_words;

   the_words: Word_List;
   input, output: Ada.Text_IO.File_Type;
begin
   Ada.Text_IO.Open(
      File => input,
      Mode => Ada.Text_IO.In_File,
      Name => "C:\Users\steve\Desktop\pla2\test1.txt"
   );
   Ada.Text_IO.Create(
      File => output,
      Mode => Ada.Text_IO.Out_File,
      Name => "C:\Users\steve\Desktop\pla2\output.txt"
   );

   get_words(input, the_words);
   sort_words(the_words);
   put_words(output, the_words);

   Ada.Text_IO.Close(input);
   Ada.Text_IO.Close(output);

   return;
end sample1;
