-- Read one word per line and print list of unique words and their frequencies
-- Case sensitive
-- This is a minimalist version.  No bells or whistles.
with ada.integer_text_io; use ada.integer_text_io;
with ada.text_io; use ada.text_io;

procedure sample2 is
   type Word is record
      s: String(1 .. 120);  -- The string.  Assume 120 characters or less
      wlen: Natural;        -- Length of the word
      count: Natural := 0;  -- Total number of occurrences of this word
   end record;

   type Word_Array is array(1 .. 1000) of Word;

   type Word_List is record
      words: Word_Array;        --  The unique words
      num_words: Natural := 0;   --  How many unique words seen so far
   end record;

   procedure split_line(s : String; wl : out Word_List) is
   temp : String(1..120);
   inWord, found : Boolean;
   ascii, word_length : Integer;
   begin
      word_length := 0;
      for i in 1 .. s'length loop
         ascii := Character'Pos(s(i));
         -- Check if at a possible first character
         if (ascii >= 65 and ascii <= 90) or (ascii >= 97 and ascii <= 122) then
            inWord := true;
         end if;

         -- Check if in a word and at a possible character
         if inWord and ((ascii >= 65 and ascii <= 90) or (ascii >= 97 and ascii <= 122) or (ascii >= 48 and ascii <= 57) or ascii = 45 or ascii = 95) then
            word_length := word_length + 1;
            temp(word_length) := s(i);

         -- Clean up at the end of a word
         elsif inWord and word_length > 0 then
            found := false;
            for i in 1 .. wl.num_words loop
               if temp = wl.words(i).s then
                  wl.words(i).count := wl.words(i).count + 1;
                  found := true;
               end if;
               exit when found;
            end loop;
            if not found then -- Add word to list
               wl.num_words := wl.num_words + 1;
               wl.words(wl.num_words).s(1 .. word_length) := temp(1 .. word_length);
               wl.words(wl.num_words).wlen := word_length;
               wl.words(wl.num_words).count := 1;
            end if;
            word_length := 0;
            inWord := false;
            temp := (others => Character'Val(0));
         end if;

      end loop;

      -- Final check for last word in line
      if inWord and word_length > 0 then
         found := false;
         for i in 1 .. wl.num_words loop
            if temp = wl.words(i).s then
               wl.words(i).count := wl.words(i).count + 1;
               found := true;
            end if;
            exit when found;
         end loop;
         if not found then -- Add word to list
            wl.num_words := wl.num_words + 1;
            wl.words(wl.num_words).s(1 .. word_length) := temp(1 .. word_length);
            wl.words(wl.num_words).wlen := word_length;
            wl.words(wl.num_words).count := 1;
         end if;
         word_length := 0;
      end if;


   end split_line;

   procedure get_words(wl: out Word_List) is
   Input : File_Type;
   begin

      -- Open the file
      Open(File => Input,
         Name => "C:\Users\steve\Desktop\pla2\test1.txt",
         Mode => In_File);

      wl.num_words := 0;  -- only to get rid of a warning
      while true loop
         declare
            s : String := Get_Line(Input);
         begin
            split_line(s, wl);

         end; --  declare
      end loop;
   exception
      when End_Error =>
         Close(Input);
   end get_words;


   -- Sorts words using selection sort for simplicity
   -- This got pretty weird to stop it from giving errors
   procedure sort_words(wl: out Word_List) is
   temp : Word;
   begin

      for i in 1 .. wl.num_words loop
         for j in i + 1 .. wl.num_words loop
            if wl.words(j).s < wl.words(i).s then

               temp := wl.words(i);
               wl.words(i) := wl.words(j);
               wl.words(j) := temp;

            end if;

         end loop;

      end loop;

   end sort_words;

   procedure put_words(wl: Word_List) is
   Output : File_Type;
   begin
      Create(File => Output, Name => "output.txt");
      for i in 1 .. wl.num_words loop
         Put_Line(Output,
         Integer'Image(wl.words(i).count) & " " & wl.words(i).s(1 .. wl.words(i).wlen));
      end loop;
      Close(Output);
   end put_words;

   the_words: Word_List;
begin
   get_words(the_words);
   sort_words(the_words);
   put_words(the_words);
end sample2;
